        .286                    ;We use 286 opcodes in section 1 for back compat
code segment byte public
        assume cs:code

;----------------------------------------------------------------------------
;
;       Autotrax XP Video driver - DOS 16 bit to Protected mode 32 bit
;
;----------------------------------------------------------------------------
;--Background:--
;Autotrax comes complete with a VGA640 driver, providing 640x480 resolution that
;seems reliable and runs on any hardware.  For those of us that want a higher
;resolution, we developed a VESA based video driver.  This has performed quite
;well, but it depends on VESA BIOS support from the video subsystem.  Back in
;WIN98 days, nearly all video cards supported VESA.  Increasingly, however,
;onboard video subsystems are cheating on VESA BIOS - it may not work completely
;Having said that, most Brand name video cards still offer VESA BIOS support, so
;there is still a method to run Autotrax with our old drivers, but at a cost.
;
;This driver does not use VESA but instead directX.  Autotrax itself, with this
;driver included, is 16 bit code.  When running under an XP/NT operating system
;this means that it cannot directly manipulate system hardware, and it cannot
;realistically just call the DirectX API.  In effect, under XP Autotrax is being
;run by a microsoft beastie called NTVDM (NT Virtual Dos Machine).  However, for
;your benefit microsoft have provided some NTVDM routines that enable us to
;build "Virtual Device Drivers" (VDD's) that can translate between 16 bit MS-DOS
;applications and "special purpose hardware" (in our case, just video, not so
;special).
;
;We write a VDD and use the NTVDM calls provided to run DirectX from Autotrax.

;--The VDD Calls:--
;The calls are so brilliantly simple it defies imagination.  I feel in awe of
;the engineer who included these in NTVDM - they are simple, basic and clean.
;Actually, I haven't said that about that big companies products before, ever.
;
;On the 16 bit app side, you just place 4 bytes in your code, for execution:
;0C4h,0C4h,58h,0Nh
;Where N selects for: 2=DispatchCall, 1=UnRegisterModule, and 0=RegisterModule
;
;At the time of calling DispatchCall or UnRegisterModule the AX register should
;be set to the VDD handle (...an identifying token...) that you obtain when you
;first call RegisterModule.  (The DispatchCall does the actual interface task.)
;
;This first call to RegisterModule also sets up the interface.  The interface is
;actually between our 16 bit DOS app and a two seperate 32 bit routines in a DLL
;When we call RegisterModule we pass pointers in registers indicating our DLL:
;DS:SI= Pointer to zero terminated ASCII string being the DLLname (autoxpdv.dll)
;ES:DI= Pointer to zero terminated ASCII string: DLL Init routine       (axinit)
;DS:BX= pointer to zero terminated ASCII string: DLL Dispatch routine   (axdisp)
;
;RegisterModule will return the VDD handle, or, if there is a problem, a code:
;AX = 1, Carry = 1:  DLL not found
;AX = 2, Carry = 1:  Dispatch routine not found
;AX = 3, Carry = 1:  Init routine not found
;AX = 4, Carry = 1:  Insufficient memory available
;AX=MMMM,Carry = 0:  Successful operation, File handle MMMM is returned
;
;By the nature of DLL's, there is actually a third routine in the 32 bit code,
;usually called LIBMAIN or DLLMAIN.  This routine is invoked when windows first
;brings up the DLL.  If it suits you, you could make the RegisterModule DLL Init
;routine redundant - and just use the LIBMAIN routine to this job.  If the
;RegisterModule DLL init routine is unused, set ES:DI to 0 in RegisterModule.
;
;The information presented here is the distilled version of what is needed to
;produce a 16 bit app to 32 bit DLL interface - google on some of the terms used
;and you will find the original information, with lots more detail (and lots of
;red herrings also - thunking and VXDs are alternative strategies we do not use)
;
;On the 32 bit side, we provide a DLL that must have 3 exposed routines as
;already mentioned - the LIBMAIN/DLLMAIN that all DLL's have, an Init routine
;(in our case axinit) and a dispatch routine (in our case axdisp).
;
;Regretfully, no data is automatically passed to our DLL.  Instead we ask NTVDM
;for the contents of the registers that were passed to it by the 16 bit app, and
;then tell NTVDM the register contents we want passed back to the 16 bit app.
;We do this through calls to "NTVDM.EXE" for functions of the style "getBX" and
;"setBX".  Each of these return or take (respectively) a 32 bit value, the lower
;16 bits of which are the register contents.  There is a seperate call for each
;of the CPU registers (eg getAX,getBX,getCX,getDX,getDI,getSI,getDS etc)
;You will need to include or declare function definitions in your DLL code.  The
;original source for these functions (if you cannot type them in) is NTVDM.LIB
;There are also functions for 8 bit and 32 bit registers, and much more besides.
;
;Credit is due to Japheth: www.japheth.de - for help and advice on the VDD calls


;Version history
;12 Nov  04 First code (but a reasonable amount of work leading up to this)
;14 Nov  04 Redoing with individual routines calling a single dispatch
;16 Nov  04 Debugging
;21 Nov  04 First version ready to go

;--Make the Jump table public--
                                ;It is just for historic value that we leave the
                                ;jump table public - the code that calls us was
                                ;compiled more than 16 years ago and frankly
                                ;wont care less what values we put here - it is
                                ;our responsibility to match the original table!
public  GETDRIVERTYPE           ;Returns 0 for Graphic
public  DRAW                    ;Arbitary x1,y1,x2,y2 line drawing algorithm
public  HLINE                   ;Draw horizontal line
public  VLINE                   ;Draw vertical line
public  PLOTPIXEL               ;
public  READPIXEL               ;Not used by any program as far as known
public  CLEARGRAPH              ;Clear screen
public  GRAPHMODE               ; vs. Textmode
public  TEXTMODE1               ;
public  FULLCHAR                ;Draw 5x7 char at X,Y; background drawn as well
public  FASTCHAR                ;Draw 5x7 char at X,Y; Do not draw background
public  SAVEMENU                ;Saves block of screen under menu image
public  LOADMENU                ;Reloads block of screen under menu image
public  XBIG                    ;Returns MaxX-1 client area or screen size
public  YBIG                    ;Returns MaxY-1 client area or screen size
public  BIGCOLORS               ;Returns number of Foreground colours, nom 15
public  BIGBACKCOLORS           ;Returns number of Background colours, nom 15
public  COLORSETS               ;Colour sets MAY be supported, - normally 0
public  ARROWCURSOR             ;Draws an Arrow shaped cursor
public  SETBACKGROUND           ;Sets the background colour
public  MENUMEMREQUIRED         ;Returns amount of memory rqd for saving menu
public  RAWPLOT                 ;
public  ARC                     ;Arc drawing routine - selectable quadrants
public  GETDRIVERNAME           ;Location of the driver title string

;--Jump Table Begins--          ;Jumps called externally: Do not move or reorder
                                ;the table, as explained above...
Start:                          ;Record module start so we can determine length
GETDRIVERTYPE:  jmp     _GetDriverType  ;Dispatch 0
DRAW:           jmp     _Draw           ;Dispatch 1
HLINE:          jmp     _Hline          ;Dispatch 2
VLINE:          jmp     _Vline          ;Dispatch 3
PLOTPIXEL:      jmp     _PlotPixel      ;Dispatch 4
READPIXEL:      jmp     _ReadPixel      ;Dispatch 5
CLEARGRAPH:     jmp     _ClearGraph     ;Dispatch 6
GRAPHMODE:      jmp     _GraphMode      ;Dispatch 7
TEXTMODE1:      jmp     _TextMode       ;Dispatch 8
FULLCHAR:       jmp     _FullChar       ;Dispatch 9
FASTCHAR:       jmp     _FastChar       ;Dispatch 10
SAVEMENU:       jmp     _SaveMenu       ;Dispatch 11
LOADMENU:       jmp     _LoadMenu       ;Dispatch 12
XBIG:           jmp     _XBig           ;Dispatch 13
YBIG:           jmp     _YBig           ;Dispatch 14
BIGCOLORS:      jmp     _BigColors      ;Dispatch 15
BIGBACKCOLORS:  jmp     _BigBackColors  ;Dispatch 16
COLORSETS:      jmp     _ColorSets      ;Dispatch 17
ARROWCURSOR:    jmp     _ArrowCursor    ;Dispatch 18
SETBACKGROUND:  jmp     _SetBackground  ;Dispatch 19
MENUMEMREQUIRED: jmp   _MenuMemRequired ;Dispatch 20
RAWPLOT:        jmp     _RawPlot        ;Dispatch 21
ARC:            jmp     _Arc            ;Dispatch 22
GETDRIVERNAME:  db      ' XP DirectX V1.01  airborn.com.au'    ;Displayed title

;--RegisterModule Routine---

Register Proc Near              ;--Invoke our service DLL with NTVDM-----
        push    ds              ;Save DS - we presume Autotrax wants it back...
        push    si
        call    SetupForLEA     ;Calculate address offset
        push    es              ;Move es segment
        pop     ds              ;  to ds
        lea     si,DLLname      ;Point to our desired drivers DLL name
        add     si,bx           ;  and fix up the offset
        lea     di,DLLinit      ;Point to the desired Init routine name
        add     di,bx           ;  and fix up the offset
        push    di              ;Save the Init routine pointer

        lea     di,DLLdisp      ;Point to the desired dispatch routine name
        add     bx,di           ;  and fix up the offset
                                ;So now we have       Dispatch in ds:bx
        pop     di              ;                     Init     in es:di
                                ;  and we already had DLLname  in ds:si

        db 0C4h,0C4h,58h,0h     ;--Execute NTVDM RegisterModule-----
                                ;We should have invoked our DLL at this point
        pop     si              ;Restore si
        pop     ds              ;Restore ds
        call    SetupforLEA     ;Calculate our address offset again
        mov   es:Handle[bx],ax  ;Save the VDD handle sent back to us by NTVDM
        mov     cx,1            ;Get a count of 1 to cx if we have good handle
        jnc     RgDone          ;Jump if we have a good VDD handle

        mov   es:Handle[bx],0   ;--ELSE we have a problem - probably no DLL----
        mov     cx,DLLFailLen   ;Get the length of the failure message
;RgDone:                        ;(This exit to print one space - had hopeed to
                                ;  encourage XP to give us focus - did not work)
        push    cx
        call    _TEXTMODE       ;Go to text mode
        pop     cx
        call    SetupforLEA     ;Calculate our address offset again

DLLFM:  mov  dl,es:DLLfail[bx]  ;Get the character to print
        mov     ah,06h          ;BIOS call 21h func 06h - output character
        int     21h             ;Send char to screen
        inc     bx              ;Move to the next character
        loop    DLLFM           ;Loop until the failure message is sent
RgDone:
        ret
Register endp

;--Dispatch routine-----
                                ;DISPATCH is the main connection point for
                                ;  functions from Autotrax thu to 32 bit DLL
Dispatch0 Proc Near             ;Enter with Dispatch code in bx; ax overwritten
                                ;We use self modifying code here for speed
        db      0B8h            ;Opcode for "mov ax,nnnn"
Handle  dw      0               ;VDD file handle; 0 means none present

        add     ax,0            ;Test the VDD handle
        jz      DispatchDone    ;  and exit if it is invalid

        db 0C4h,0C4h,58h,2h     ;--Execute NTVDM DispatchCall-----
DispatchDone:
        ret
Dispatch0 endp



;--Original Autotrax Function routines now coded to call Dispatch-----

_GetDriverType proc NEAR        ;--Return code for our driver type (0)---
                                ;0 =Graphics  1 =Printer  2 =Plotter  3 =Locator
        mov     bx,0            ;Dispatch code 0 for get driver type
        call    Dispatch0       ;Send the function to the 32 bit DLL
        mov     ax,0
        ret
_GetDriverType endp


_Hline Proc Near                ;--Draw a horizontal line between 2 points-----
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+10]      ;Get x1                 --> cx
        mov     dx,[bp+6]       ;Get y1 (same as y2)    --> dx
        mov     si,[bp+8]       ;Get x2                 --> si
        mov     di,dx           ;Get y2                 --> di
        mov     bp,[bp+4]       ;Get Colour             --> bp

        mov     bx,2            ;Dispatch code 2 for Horiztonal line
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     8
_Hline endp


_Vline Proc Near                ;--Draw a vertical line between 2 points-----
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+10]      ;Get x1                 --> cx
        mov     dx,[bp+8]       ;Get y1                 --> dx
        mov     si,cx           ;Get x2 (same as x1)    --> si
        mov     di,[bp+6]       ;Get y2                 --> di
        mov     bp,[bp+4]       ;Get Colour             --> bp

        mov     bx,3            ;Dispatch code 3 for Vertical line
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     8
_Vline endp


_ReadPixel Proc Near            ;--Return the colour of the Pixel at x,y------
                                ;In test 14 Nov 2004 this function seemed unused
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+6]       ;Get x1                 --> cx
        mov     dx,[bp+4]       ;Get y1                 --> dx

        mov     bx,5            ;Dispatch code 5 for Read Pixel
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     6               ;Note ret 6 - seems to want to pop 1 extra parm
_ReadPixel endp


_ClearGraph Proc Near           ;--Clear the screen to background colour-----
        push    bp
        mov     bx,6            ;Dispatch code 6 for ClearGraph
        call    Dispatch0       ;Send the function to the 32 bit DLL
        call    SetGraph
        pop     bp
        ret
_ClearGraph endp


_GraphMode Proc Near            ;--Init Graphics mode-----
;        call    Register
        mov     bx,7            ;Dispatch code 7 for Init Graphics mode
        call    Dispatch0       ;Send the function to the 32 bit DLL
        call    SetGraph
        ret
_GraphMode endp


_TextMode Proc Near             ;--Jump to text mode (vs Graphics mode)-----
        mov     ax,0003h        ;Ask for "Video Mode 3": 80x25 16 colours
                                ;This mode was the default power up mode for
                                ;  any DOS system with a colour monitor
        int     10h
        mov     bx,8            ;Dispatch code 8 for TextMode function
        call    Dispatch0       ;Send the code to AxDispatch if possible
        ret
_TextMode endp


_SaveMenu Proc Near             ;--Save the 320pix x 200line menu space-----
                                ;We are told to save top left screen area to mem
                                ;We are provided with the memory space - but for
                                ;  speed we instead use space in our 32 bit DLL
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+6]       ;Redundant fetch of Segm--> cx
        mov     dx,[bp+4]       ;Redundant fetch of Offs--> dx

        mov     bx,11           ;Dispatch code of 11 for SaveMenu
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     4
_SaveMenu endp


_LoadMenu Proc Near             ;--Restore the 320pox x 200line menu space-----
                                ;We are told to restore the top left screen area
                                ;  from a memory buffer where we previously
                                ;  saved that data - See _SaveMenu
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+6]       ;Segment of mem area provided by 16 bit app ->cx
        mov     dx,[bp+4]       ;Offset  of mem area provided by 16 bit app ->dx

        mov     bx,12           ;Dispatch code of 12 for LoadMenu
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     4
_LoadMenu endp


_XBig proc Near                 ;Returns X size
        mov     bx,13           ;Dispatch code 13 Max X value
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
_XBig endp


_YBig proc Near                 ;Returns Y size
        call    Register
        mov     bx,14           ;Dispatch code 14 Max Y value
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
_YBig endp


_BigColors proc Near            ;Returns number of colours available
                                ;Some info says must not be larger than 16
                                ;Basic test on 17 Nov 2004 suggests 31 is OK
        mov     bx,15           ;Dispatch code of 15 for number of colours
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
_BigColors endp


_BigBackColors proc Near        ;Returns number of colours avail for background
                                ;Some info says must not be larger than 16
        mov     bx,16           ;Dispatch code of 16 for number of bkgnd colours
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
_BigBackColors endp


_ColorSets proc Near            ;Returns the number of color sets supported
                                ;For example the CGA driver has 2 set of 4 cols
        mov     bx,17           ;Dispatch code of 17 for number of colour sets
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
_ColorSets endp


_ArrowCursor Proc Near          ;--Draws an arrow cursor at location x,y-----
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     cx,[bp+6]       ;Get x1                 --> cx
        mov     dx,[bp+4]       ;Get y1                 --> dx
        mov     si,0FFFFh

        mov     bx,18           ;Dispatch code of 18 for ArrowCursor
        call    Dispatch0       ;Send the function to the 32 bit DLL
        call    SetGraph

        pop     bp
        ret     4
_ArrowCursor endp


_SetBackground Proc Near        ;--Set the background colour-----
        push    bp
        mov     bp,sp           ;Address the stack frame for parameters

        mov     di,[bp+4]       ;Get ColourSet (unused) --> di
        mov     si,[bp+6]       ;Set si to Colour, also
        mov     bp,[bp+6]       ;Get Colour             --> bp

        mov     bx,19           ;Dispatch code 19 for set background colour
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bp
        ret     4
_SetBackground endp


_MenuMemRequired proc NEAR      ;Returns the amount of memory required for menu
        mov     bx,20           ;Dispatch code of 20 for menu memory
        call    Dispatch0       ;Send the function to the 32 bit DLL
       ' mov     ax,0
        ret
_MenuMemRequired endp














;--Internal subroutines-----


SetGraph Proc Near              ;--Any setups required prior to Graph draw----
        mov     bx,23           ;Dispatch code 23 for SetGraph function
        call    Dispatch0       ;Send the function to the 32 bit DLL
        call    SetupForLEA     ;Jump to setup segment reg and Bx
        ret
SetGraph endp


ResetGraph Proc Near            ;--Any takedowns required after Graph draw----
        mov     bx,24           ;Dispatch code 24 for ResetGraph function
        call    Dispatch0       ;Send the function to the 32 bit DLL
        ret
ResetGraph endp


Point Proc Near                 ;--Set pixel X,Y to colour-----
                                ;Routine AVOIDS plotting pixels over status line
                                ;If supporting ArrowCursor, push: bx,cx,dx
        push    ax
        push    bx
                                ;X Co-ordinate is already in cx
                                ;Y Co-ordinate is already in dx
        mov     bx,25           ;Dispatch code 25 for Point function
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bx
        pop     ax
        ret
Point endp


TextPoint Proc Near             ;--Set pixel X,Y to colour-----
                                ;Routine ONLY plotting pixels over status line
        push    ax
        push    bx
                                 ;X Co-ordinate is already in cx
                                 ;Y Co-ordinate is already in dx
        mov     bx,26           ;Dispatch code 26 for TextPoint function
        call    Dispatch0       ;Send the function to the 32 bit DLL

        pop     bx
        pop     ax
        ret
TextPoint endp


SetupForLEA Proc Near           ;--Find offset address--------
        call    OurCod9         ;Find our current location
OurCod9: pop    bx
        sub  bx,offset(OurCod9) ;bx points to the start of section
        push    cs              ;Copy code segment
        pop     es              ;  into es
        ret
SetupForLEA endp




;--gcommon.asm import---------------------------------------------------------
                                ;GCOMMON.ASM is the original protel common code
                                ;currently we import and support this, it may
                                ;possibly be supplanted, incorporated, removed
                                ;  Calls: SetGraph,ResetGraph,Point,TextPoint

                                ;Currenly in gcommon.asm: _GetDrivertype:
                                ;Currenly in gcommon.asm: _Draw:
                                ;Currenly in gcommon.asm: _PlotPixel:
                                ;Currenly in gcommon.asm: _FullChar:
                                ;Currenly in gcommon.asm: _FastChar:
                                ;Currenly in gcommon.asm: _XBig:
                                ;Currenly in gcommon.asm: _YBig:
                                ;Currenly in gcommon.asm: _BigColors:
                                ;Currenly in gcommon.asm: _BigBackColors:
                                ;Currenly in gcommon.asm: _ColorSets:
                                ;Currenly in gcommon.asm: _MenuMemRequired:
                                ;Currenly in gcommon.asm: _RawPlot:
                                ;Currenly in gcommon.asm: _Arc:


;********************************************;
_PLOTPIXEL proc NEAR                         ;
; Turn on the pixel at (px,py) and set it to ;
; color Pcolor                               ;
;********************************************;
px	equ	[bp+8]
py	equ	[bp+6]
pcolor 	equ	[bp+4]

	push	bp
	mov	bp,sp
	mov	si,pcolor
        call    SetGraph
	mov	Dx,py
	mov	Cx,px

	call	Point
	call	ResetGraph
	pop	bp
	ret     6
_PLOTPIXEL endp


;*************************************************;
_RAWPLOT proc Near                                ;
; Point plotting routine called by other routines ;
; in the high level and other ASM modules.        ;
; X = Cx                                          ;
; Y = Dx                                          ;
; SI = color                                      ;
; called for bitmap routine in pascal zoom := 1   ;
;*************************************************;
      push    ax
      push    bx
      push    cx
      push    dx

      Call    SetGraph
      call    point
      call    ResetGraph

      pop     dx
      pop     cx
      pop     bx
      pop     ax
      ret
_RAWPLOT endp


;*********************************************;
_DRAW proc NEAR                               ;
; Use Bresenhams line drawing algorithmn do   ;
; draw a line between (x1,y1) and (x2,y2). No ;
; assumptions about the slope of the line are ;
; made.                                       ;
;Procedure Draw(x1,y1,x2,y2,color:integer);   ;
;*********************************************;
x1	equ	[bp+12]
y1	equ	[bp+10]
x2	equ	[bp+8]
y2	equ	[bp+6]
dcolor	equ	[bp+4]
deltax	equ	[bp-2]
deltay	equ	[bp-4]
incr1	equ	[bp-6]
incr2	equ	[bp-8]
incr3	equ	[bp-10]
xend	equ	[bp-12]
yend	equ	[bp-14]

	push	bp
	mov	bp,sp
	sub	sp,14

	mov	si,dcolor	;si = color
        call    SetGraph
	mov	ax,y2		;get y2
	sub	ax,y1		;y2 := y2 - y1
	jns	noychang	;jump if y2 > y1
	neg	ax
noychang:
	mov	deltay,ax	;save deltay

	mov	ax,x2		;get x2
	sub	ax,x1		;x2 := x2 - x1
	jns	noxchang	;jump if x2 > x1
	neg	ax
noxchang:
	mov	deltax,ax	;save deltax
	cmp	ax,deltay	;jump if abs(deltax) > abs(deltay)
	jg	firstLoop
	jmp	secLoop

FirstLoop:
	mov	ax,x1
	cmp	ax,x2		;jump if x1 <= x2
	jle	biggerx
	mov	cx,x2		;x := x2
	mov	dx,y2		;y := y2
	mov	ax,x1
	mov	Xend,ax		;xend := x1
	mov	ax,y1
	sub	ax,y2		;Deltay = y1-y2
	mov	Deltay,ax
	Jmp short Fin1
biggerx:
	mov	cx,x1		;x = x1
	mov	dx,y1		;y = y1
	mov	ax,x2
	mov	Xend,ax		;xend := x2
	mov	ax,y2
	sub	ax,y1		;Deltay = y2-y1
	mov	Deltay,ax
Fin1:
	shl	ax,1
	mov	incr1,ax	;Incr1 := (deltay shl 1)
	sub	ax,Deltax
	mov	di,ax		;Dirc := (deltay shl 1) - deltax
	mov	ax,deltay
	sub	ax,deltax
	shl	ax,1
	mov	incr2,ax	;incr2 := (deltay-deltax) shl 1
	mov	ax,deltay
	add	ax,deltax
	shl	ax,1
	mov	incr3,ax	;incr3 := (deltay+deltax) shl 1

	call	point
main:
	cmp	cx,xend
	jge	finishF		;jump if x >= xend

	inc	cx		;x := x + 1
	cmp	di,0
	jl	pt2		;jump if dirc < 0

pt1:	mov	ax,deltay
	cmp	ax,0		;Jump if deltay > 0
	jg	part11
	add	di,incr1
	Jmp short endloop
part11:
	inc	dx		;y := y +1;
	add	di,incr2
	Jmp short endloop

pt2:	mov	ax,deltay
	cmp	ax,0		;Jump if deltay < 0
	jl	part21
	add	di,incr1	;dirc := dirc + incr1
	Jmp short endloop
part21:
	dec	dx		;y := y - 1;
	add	di,incr3	;dirc := dirc + incr3
endloop:
	call	point
	Jmp short main

FinishF:
	jmp	Finish

SecLoop:
	mov	ax,y1		;ax = y1
	cmp	ax,y2		;jump	if y1 <= y2
	jle	biggerx2
	mov	cx,x2  		;x := x2
	mov	dx,y2  		;y := y2
	mov	ax,y1
	mov	Yend,ax		;Yend := Y1
	mov	ax,x1
	sub	ax,x2  		;Deltax = x1-x2
	mov	Deltax,ax
	Jmp short Fin2
biggerx2:
	mov	cx,x1  		;x = x1
	mov	dx,y1  		;y = y1
	mov	ax,y2
	mov	Yend,ax		;Yend := Y2
	mov	ax,x2
	sub	ax,x1  		;Deltax = x2-x1
	mov	Deltax,ax
Fin2:
	shl	ax,1
	mov	incr1,ax   	;Incr1 := (deltax shl 1)
	sub	ax,Deltay
	mov	di,ax		;Dirc := (deltax shl 1) - deltay
	mov	ax,deltax
	sub	ax,deltay
	shl	ax,1
	mov	incr2,ax   	;incr2 := (deltax-deltay) shl 1
	mov	ax,deltax
	add	ax,deltay
	shl	ax,1
	mov	incr3,ax   	;incr3 := (deltax+deltay) shl 1

	call	point
main2:
	cmp	dx,yend
	jge	finish		;jump if y >= yend

	inc	dx		;y := y + 1
	cmp	di,0
	jl	Spt2		;jump if dirc < 0

Spt1: mov	ax,deltax
	cmp	ax,0		;Jump if deltax > 0
	jg	Spart11
	add	di,incr1
	Jmp short endloop2
Spart11:
	inc	cx		;x := x +1;
	add	di,incr2
	Jmp short endloop2

Spt2: mov	ax,deltax
	cmp	ax,0		;Jump if deltax < 0
	jl	Spart21
	add	di,incr1   	;dirc := dirc + incr1
	Jmp short endloop2
Spart21:
	dec	cx		;x := x - 1;
	add	di,incr3   	;dirc := dirc + incr3
endloop2:
	call	point
	Jmp short main2

finish:
	call	ResetGraph
	mov	sp,bp
	pop	bp
	ret     10
_DRAW endp

;************************************************;
_FASTCHAR proc Near                              ;
; Draws a 5x7 pixel character at ftx,fty.        ;
; ftseg and ftofs specify the memory location of ;
; the bitmap data. Only active pixels are drawn. ;
; if ftorien is 0 the character is horizontal, if;
; not then draw it vertical.                     ;
; The ftch is the character.                     ;
; This routine is device independant except for  ;
; the ResetGraph Call and relies on the TextPoint;
; sub-routine to be correct.                     ;
;************************************************;
ftorien	equ	[bp+18]
ftseg	equ	[bp+16]
ftofs	equ	[bp+14]
ftch	equ	[bp+12]
ftx	equ	[bp+10]
fty	equ	[bp+8]
ftcolor	equ	[bp+6]
	push	ds
	push	bp
	mov	bp,sp

	mov	si,ftcolor	;si = color
        call    SetGraph

	mov	bx,ftofs	;get Offset
	mov	ax,ftseg	;get seg	position of text array
	mov	ds,ax
	mov	ax,ftorien	;get orien 0 = hor 1 = ver
	or	ax,ax
	jz	Horizontal
	jmp	vertical

Horizontal:
	mov	cx,ftx		;get x
	inc	cx
	inc	cx
	mov	ftx,cx		;put new (missing first 2 bits)
	mov	dx,fty		;get y
	inc	dx		;top line blank
	mov	fty,dx		;put y


	mov	ax,ftch		;get character
	and	ax,127
	sub	ax,32		;normalize for dc
	shl	ax,1
	shl	ax,1
	shl	ax,1
	inc	ax		;first character always blank
	add	bx,ax		;bx now points at first byte in char
	mov	di,8		;8 lines in character

mainLoop:
	mov	al,ds:0[bx]	;Get Byte
	or	al,al		;if zero then add 8 to x and jump to end
	jz	Empty

NotEmpty:
	test	al,020H		;if bit set then plot
	jz	test1
	call	point		;plot it
test1:
	Inc	cx		;x := x + 1;
	test	al,010H
	jz	test2
	call	point		;plot it
test2:
	Inc	cx		;x := x + 1;
	test	al,08
	jz	test3
	call	point		;plot it
test3:
	Inc	cx		;x := x + 1;
	test	al,04
	jz	test4
	call	point		;plot it
test4:
	Inc	cx		;x := x + 1;
	test	al,02
	jz	empty
	call	point		;plot it
empty:
	Inc	bx		;Point at next byte in array;
	mov	cx,ftx		;get x back
	inc	dx		;y := y + 1;

	dec	di		;decrement line count
	jnz	mainloop

	mov	ax,ftch		;get character back
	test	ax,80h		;if high bit set then invert line
	jnz	doit1
	jmp	endit
doit1:
	mov	cx,ftx		;get x
	dec	cx
	dec	cx
	mov	dx,fty		;get y
	dec	dx
	dec	dx
	call	point		;plot(x,y-2)
	inc	cx
	call	point		;plot(x+1,y-2)
	inc	cx
	call	point		;plot(x+2,y-2)
	inc	cx
	call	point		;plot(x+3,y-2)
	inc	cx
	call	point		;plot(x+4,y-2)
	inc	cx
	call	point		;plot(x+5,y-2)
	inc	cx
	call	point		;plot(x+6,y-2)
	inc	cx
	call	point		;plot(x+7,y-2)
	jmp	endit

Vertical:
	mov	dx,fty		;get y
	dec	dx
	dec	dx
	mov	fty,dx		;put new y(missing first 2 bits)
	mov	cx,ftx		;get x
	dec	cx		;top line blank
	mov	ftx,cx		;put x

	mov	ax,ftch		;get character
	and	ax,127
	sub	ax,32		;normalize for dc

	shl	ax,1
	shl	ax,1
	shl	ax,1
	inc	ax		;first character always blank
	add	bx,ax		;bx now points at first byte in char
	mov	di,8		;8 lines in character

mainLoopV:
	mov	al,ds:0[bx]	;Get Byte
	or	al,al		;if zero then add 8 to x and jump to end
	jz	EmptyV

NotEmptyV:
	test	al,020H		;if bit set then plot
	jz	testV1
	call	point		;plot it
testV1:
	dec	dx		;y := y + 1;
	test	al,010H
	jz	testV2
	call	point		;plot it
testV2:
	dec	dx		;y := y + 1;
	test	al,08
	jz	testV3
	call	point		;plot it
testV3:
	dec	dx		;y := y + 1;
	test	al,04
	jz	testV4
	call	point		;plot it
testV4:
	dec	dx		;y := y + 1;
	test	al,02
	jz	emptyV
	call	point		;plot it
emptyV:
	Inc	bx		;Point at next byte in array;
	mov	dx,fty 		;get y back
	inc	cx		;x := x + 1;

	dec	di		;decrement line count
	jnz	mainloopV

	mov	ax,ftch		;get character back
	test	ax,80h		;if high bit set then invert line
	jz	endit

	mov	dx,fty 		;get y
	inc	dx
	inc	dx
	mov	cx,ftx 		;get x
	dec	cx
	dec	cx
	call	point		;plot(x-2,y-2)
	dec	dx
	call	point		;plot(x+1,y-2)
	dec	dx
	call	point		;plot(x+2,y-2)
	dec	dx
	call	point		;plot(x+3,y-2)
	dec	dx
	call	point		;plot(x+4,y-2)
	dec	dx
	call	point		;plot(x+5,y-2)
	dec	dx
	call	point		;plot(x+6,y-2)
	dec	dx
	call	point		;plot(x+7,y-2)
	jmp short endit
endit:

	call	ResetGraph
	pop	bp
	pop	ds
	ret     14
_FASTCHAR endp


;************************************************;
_FULLCHAR proc Near                              ;
; Draws a 5x7 pixel character at btx,bty.        ;
; btseg and btofs specify the memory location of ;
; the bitmap data. All pixel (active or passive  ;
; are drawn.                                     ;
; Passive pixels are drawn in the btback color,  ;
; active pixels are drawn in the btcolor color   ;
; The ftch is the character.                     ;
; This routine is device independant except for  ;
; the ResetGraph Call and relies on the TextPoint;
; sub-routine to be correct.                     ;
;************************************************;
btback	equ	[bp+18]
btseg	equ	[bp+16]
btofs	equ	[bp+14]
btch	equ	[bp+12]
btx	equ	[bp+10]
bty	equ	[bp+8]
btcolor	equ	[bp+6]

	push	ds
	push	bp
	mov	bp,sp

        mov     si,btcolor
        call    Setgraph
	mov	bx,btofs	;get Offset
	mov	ax,btseg	;get seg	position of text array
	mov	ds,ax

	mov	cx,btx		;get x
	inc	cx
	inc	cx
	mov	btx,cx		;put new (missing first 2 bits)
	mov	dx,bty		;get y
	sub	dx,8

	mov	ax,btch		;get character
	and	ax,127
	sub	ax,32		;normalize for dc
	shl	ax,1
	shl	ax,1
	shl	ax,1
	inc	ax		;first Line of character always blank
	add	bx,ax		;bx now TextPoints at first byte in char
	mov	di,8		;8 lines in character

TmainLoop:
	mov	al,ds:0[bx]	;Get Byte
	Mov	si,btback
	test	al,020H		;if bit set then plot
	jz	Plot1
	mov	si,btcolor
Plot1:
	call	TextPoint

	Inc	cx
	Mov	si,btback
	test	al,010H
	jz	Plot2
	mov	si,btcolor
Plot2:
	call	TextPoint

	Inc	cx
	Mov	si,btback
	test	al,08
	jz	Plot3
	mov	si,btcolor
Plot3:
	call	TextPoint

	Inc	cx
	Mov	si,btback
	test	al,04
	jz	Plot4
	mov	si,btcolor
Plot4:
	call	TextPoint

	Inc	cx
	Mov	si,btback
	test	al,02
	jz	Plot5
	mov	si,btcolor
Plot5:
	call	TextPoint

	Inc	bx
	mov	cx,btx
	inc	dx

	dec	di
	jnz	Tmainloop

	call	ResetGraph
	pop	bp
	pop	ds
	ret     14
_FULLCHAR endp



;*********************************************;
_Arc proc NEAR                                ;
; Use Bresenhams circle drawing algorithmn    ;
; procedure Arc(xc,yc,segments,radius,color)  ;
;*********************************************;
xc	equ	[bp+12]
yc	equ	[bp+10]
segs    equ	[bp+8]
radius	equ	[bp+6]
acolor	equ	[bp+4]

		push	bp
		mov		bp,sp

		mov		si,dcolor	       ;si = color
        call    SetGraph

        mov     cx,0                   ;x := 0;
        mov     dx,radius              ;y := radius;

        mov     di,3
        sub     di,radius              ;di := 3 - 2 * Radius;
        sub     di,radius

A_MainArcLoop:
        cmp     cx,dx
        jle     A_DoArc                 ;while x <= y do
        jmp     A_EndArc
A_DoArc:
        push    cx
        push    dx
        mov     ax,cx                 ;ax = x offset
        mov     bx,dx                 ;bx = y offset

A_Seg2:
		test    word ptr segs,2
        jz      A_seg3

        mov     cx,xc
        sub     cx,bx
        mov     dx,yc
        sub     dx,ax
        call    point                  ;Plot(xc-y,yc-x)

A_Seg3:
		test    word ptr segs,1
        jz      A_seg4

        mov     cx,xc
        add     cx,bx
        mov     dx,yc
        sub     dx,ax
        call    point                  ;Plot(xc+y,yc-x)
A_Seg4:
        pop     dx
        pop     cx

        or      di,di                   ;if di < 0 then
        jns     A_digreatz
        mov     ax,cx                   ;di := di + (4 * x) + 6;
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,6
        jmp     A_endtest

A_digreatz:
        mov     ax,cx
        sub     ax,dx                   ;di := di + 4 * (x-y) + 10;
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,10
        dec     dx
A_endtest:
        inc     cx
        jmp     A_MainArcLoop
A_EndArc:
        mov     cx,0                   ;x := 0;
        mov     dx,radius              ;y := radius;

        mov     di,3
        sub     di,radius              ;di := 3 - 2 * Radius;
        sub     di,radius

B_MainArcLoop:
        cmp     cx,dx
        jle     B_DoArc                 ;while x <= y do
        jmp     B_EndArc
B_DoArc:
        push    cx
        push    dx
        mov     ax,cx                 ;ax = x offset
        mov     bx,dx                 ;bx = y offset

B_Seg2:
		test    word ptr segs,2
        jz      B_seg3

        mov     cx,xc
        sub     cx,ax
        mov     dx,yc
        sub     dx,bx
        call    point                  ;Plot(xc-x,yc-y)

B_Seg3:
		test    word ptr segs,1
        jz      B_seg4
; top right
        mov     cx,xc
        add     cx,ax
        mov     dx,yc
        sub     dx,bx
        call    point                  ;Plot(xc+x,yc-y)

B_Seg4:
        pop     dx
        pop     cx

        or      di,di                   ;if di < 0 then
        jns     B_digreatz
        mov     ax,cx                   ;di := di + (4 * x) + 6;
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,6
        jmp     B_endtest

B_digreatz:
        mov     ax,cx
        sub     ax,dx                   ;di := di + 4 * (x-y) + 10;
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,10
        dec     dx
B_endtest:
        inc     cx
        jmp     B_MainArcLoop
B_EndArc:
        mov     cx,0                   ;x := 0;
        mov     dx,radius              ;y := radius;

        mov     di,3
        sub     di,radius              ;di := 3 - 2 * Radius;
        sub     di,radius

C_MainArcLoop:
        cmp     cx,dx
        jle     C_DoArc                 ;while x <= y do
        jmp     C_EndArc
C_DoArc:
        push    cx
        push    dx
        mov     ax,cx                 ;ax = x offset
        mov     bx,dx                 ;bx = y offset

        test    word ptr segs,8
        jz      C_seg1
        mov     cx,xc
        add     cx,bx
        mov     dx,yc
        add     dx,ax
        call    point                  ;Plot(xc+y,yc+x)

C_Seg1:
		test    word ptr segs,4
        jz      C_seg4
        mov     cx,xc
        sub     cx,bx
        mov     dx,yc
        add     dx,ax
        call    point                  ;Plot(xc-y,yc+x)

C_Seg4:
        pop     dx
        pop     cx

        or      di,di                   ;if di < 0 then
        jns     C_digreatz
        mov     ax,cx                   ;di := di + (4 * x) + 6;
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,6
        jmp     C_endtest

C_digreatz:
        mov     ax,cx
        sub     ax,dx                   ;di := di + 4 * (x-y) + 10;
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,10
        dec     dx
C_endtest:
        inc     cx
        jmp     C_MainArcLoop
C_EndArc:
        mov     cx,0                   ;x := 0;
        mov     dx,radius              ;y := radius;

        mov     di,3
        sub     di,radius              ;di := 3 - 2 * Radius;
        sub     di,radius

D_MainArcLoop:
        cmp     cx,dx
        jle     D_DoArc                 ;while x <= y do
        jmp     D_EndArc
D_DoArc:
        push    cx
        push    dx
        mov     ax,cx                 ;ax = x offset
        mov     bx,dx                 ;bx = y offset

        test    word ptr segs,8
        jz      D_seg1
; bottom right
        mov     cx,xc
        add     cx,ax
        mov     dx,yc
        add     dx,bx
        call    point                  ;Plot(xc+x,yc+y)

D_Seg1:   
		test    word ptr segs,4
        jz      D_seg4
; bottom left
        mov     cx,xc
        sub     cx,ax
        mov     dx,yc
        add     dx,bx
        call    point                  ;Plot(xc-x,yc+y)

D_Seg4:
        pop     dx
        pop     cx

        or      di,di                   ;if di < 0 then
        jns     D_digreatz
        mov     ax,cx                   ;di := di + (4 * x) + 6;
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,cx
        add     di,6
        jmp     D_endtest

D_digreatz:
        mov     ax,cx
        sub     ax,dx                   ;di := di + 4 * (x-y) + 10;
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,ax
        add     di,10
        dec     dx
D_endtest:
        inc     cx
        jmp     D_MainArcLoop
D_EndArc:


EndArc:
        call	ResetGraph
        pop     bp
        ret     10

_ARC    endp


;------------------------------------------------------------------------------
;Please start the message below with a space character:
DLLfail db      ' (You will need to push "Esc Esc F Q Y" after reading this:)'
        db      0Dh,0Ah,'The Autotrax XP video driver could not initialize '
        db      'correctly, the reason:',0Dh,0Ah
        db      'File path incorrect or required file not found: '
DLLname db      'AUTOXPDV.DLL',0
DLLFailLen   =  $ - DLLfail
DLLinit db      'AxInit',0
DLLdisp db      'AxDisp',0

;Insert Steves favourite quote:
db '...I do not know whether I was then a man dreaming I was a butterfly, '
db 'or whether I am now a butterfly dreaming I am a man.  - Chuang Tzu '

CodeLength   =  $ - Start
db 4096-CodeLength DUP(0)       ;Fill the image to make a length of 4k bytes

code ends
     end
